@@ -166,11 +166,8 @@ dySeries <- function(dygraph,
   attrs$series[[series$label]] <- series$options
   
   # add color if specified 
-  if (!is.null(color)) {
-    if (length(color) == 1)
-      color[[2]] <- color[[1]]
-    attrs$colors <- c(attrs$colors, color)
-  }
+  if (!is.null(color))
+    attrs$colors <- as.list(c(attrs$colors, color))
   
   # set attrs
   dygraph$x$attrs <- attrs
