@@ -1,3 +1,26 @@
+
+// polyfill indexOf for IE8
+if (!Array.prototype.indexOf) {
+  Array.prototype.indexOf = function(elt /*, from*/) {
+    var len = this.length >>> 0;
+
+    var from = Number(arguments[1]) || 0;
+    from = (from < 0)
+         ? Math.ceil(from)
+         : Math.floor(from);
+    if (from < 0)
+      from += len;
+
+    for (; from < len; from++) {
+      if (from in this &&
+          this[from] === elt)
+        return from;
+    }
+    return -1;
+  };
+}
+
+
 HTMLWidgets.widget({
 
   name: "dygraphs",
@@ -46,11 +69,11 @@ HTMLWidgets.widget({
     
     // convert time to js time
     attrs.file[0] = attrs.file[0].map(function(value) {
-      return thiz.normalizeDateValue(x.scale, value);
+      return thiz.normalizeDateValue(x.scale, value, x.fixedtz);
     });
     if (attrs.dateWindow != null) {
       attrs.dateWindow = attrs.dateWindow.map(function(value) {
-        var date = thiz.normalizeDateValue(x.scale, value);
+        var date = thiz.normalizeDateValue(x.scale, value, x.fixedtz);
         return date.getTime();
       });
     }
@@ -65,17 +88,25 @@ HTMLWidgets.widget({
     // add shading and event callback if necessary
     this.addShadingCallback(x);
     this.addEventCallback(x);
+    this.addZoomCallback(x, instance);
       
-    // add default font for viewer mode
-    if (this.queryVar("viewer_pane") === "1")
-      document.body.style.fontFamily = "Arial, sans-serif";
     
-    if (instance.dygraph) { // update existing instance
-       
-      instance.dygraph.updateOptions(attrs);
-    
-    } else {  // create new instance
+    // if there is no existing instance perform one-time initialization
+    if (!instance.dygraph) {
       
+      // subscribe to custom shown event (fired by ioslides to trigger
+      // shiny reactivity but we can use it as well). this is necessary
+      // because if a dygraph starts out as display:none it has height
+      // and width == 0 and this doesn't change when it becomes visible
+      $(el).closest('slide').on('shown', function() {
+        if (instance.dygraph)
+          instance.dygraph.resize();  
+      });
+      
+      // add default font for viewer mode
+      if (this.queryVar("viewer_pane") === "1")
+        document.body.style.fontFamily = "Arial, sans-serif";
+
       // add shiny input for date window
       if (HTMLWidgets.shinyMode)
         this.addDateWindowShinyInput(el.id, x);
@@ -91,17 +122,35 @@ HTMLWidgets.widget({
         document.getElementsByTagName("head")[0].appendChild(style);
       }
       
-      // create the instance and add it to it's group (if any)
-      instance.dygraph = new Dygraph(el, attrs.file, attrs);
-      if (x.group != null)
-        this.groups[x.group].push(instance.dygraph);
+    } else {
+      
+        // retain the userDateWindow
+        if (instance.dygraph.userDateWindow != null)
+          attrs.dateWindow = instance.dygraph.xAxisRange();
+      
+        // remove it from groups if it's there
+        if (x.group != null && this.groups[x.group] != null) {
+          var index = this.groups[x.group].indexOf(instance.dygraph);
+          if (index != -1)
+            this.groups[x.group].splice(index, 1);
+        }
+        
+        // destroy the existing dygraph 
+        instance.dygraph.destroy();
+        instance.dygraph = null;
     }
-     
+    
+    // create the instance and add it to it's group (if any)
+    instance.dygraph = new Dygraph(el, attrs.file, attrs);
+    instance.dygraph.userDateWindow = attrs.dateWindow;
+    if (x.group != null)
+      this.groups[x.group].push(instance.dygraph);
+    
     // set annotations
     if (x.annotations != null) {
       instance.dygraph.ready(function() {
         x.annotations.map(function(annotation) {
-          var date = thiz.normalizeDateValue(x.scale, annotation.x);
+          var date = thiz.normalizeDateValue(x.scale, annotation.x, x.fixedtz);
           annotation.x = date.getTime();
         });
         instance.dygraph.setAnnotations(x.annotations);
@@ -237,7 +286,7 @@ HTMLWidgets.widget({
         return mmnt.format('YYYY');
       }else{
         if(granularity >= Dygraph.MONTHLY){
-          return mmnt.format('MMM YY');
+          return mmnt.format('MMM YYYY');
         }else{
           var frac = mmnt.hour() * 3600 + mmnt.minute() * 60 + mmnt.second() + mmnt.millisecond();
             if (frac === 0 || granularity >= Dygraph.DAILY) {
@@ -262,11 +311,11 @@ HTMLWidgets.widget({
         if (scale == "yearly")
           return mmnt.format('YYYY') + ' (' + mmnt.zoneAbbr() + ')';
         else if (scale == "monthly" || scale == "quarterly")
-          return mmnt.format('MMMM, YYYY')+ ' (' + mmnt.zoneAbbr() + ')';
+          return mmnt.format('MMM, YYYY')+ ' (' + mmnt.zoneAbbr() + ')';
         else if (scale == "daily" || scale == "weekly")
-          return mmnt.format('MMMM, DD, YYYY')+ ' (' + mmnt.zoneAbbr() + ')';
+          return mmnt.format('MMM, DD, YYYY')+ ' (' + mmnt.zoneAbbr() + ')';
         else
-          return mmnt.format('MMMM, DD, YYYY HH:mm:ss')+ ' (' + mmnt.zoneAbbr() + ')';
+          return mmnt.format('dddd, MMMM, DD, YYYY HH:mm:ss')+ ' (' + mmnt.zoneAbbr() + ')';
     }
   },
   
@@ -280,16 +329,52 @@ HTMLWidgets.widget({
         if (scale == "yearly")
           return date.getFullYear();
         else if (scale == "monthly" || scale == "quarterly")
-          return monthNames[date.getMonth()] + ' ' + date.getFullYear(); 
+          return monthNames[date.getMonth()] + ', ' + date.getFullYear(); 
         else if (scale == "daily" || scale == "weekly")
-          return monthNames[date.getMonth()] + ' ' + 
-                           date.getDate() + ' ' + 
+          return monthNames[date.getMonth()] + ', ' + 
+                           date.getDate() + ', ' + 
                            date.getFullYear();
         else
           return date.toLocaleString();
     }
   },
   
+  addZoomCallback: function(x, instance) {
+    
+    // alias this
+    var thiz = this;
+    
+    // get attrs
+    var attrs = x.attrs;
+    
+    // check for an existing zoomCallback
+    var prevZoomCallback = attrs["zoomCallback"];
+    
+    attrs.zoomCallback = function(minDate, maxDate, yRanges) {
+      
+      // call existing
+      if (prevZoomCallback)
+        prevZoomCallback(minDate, maxDate, yRanges);
+        
+      // record user date window (or lack thereof)
+      var me = instance.dygraph;
+      if (me.xAxisExtremes()[0] != minDate ||
+          me.xAxisExtremes()[1] != maxDate) {
+         me.userDateWindow = [minDate, maxDate];
+      } else {
+         me.userDateWindow = null;
+      }
+      
+      // record in group if necessary
+      if (x.group != null && thiz.groups[x.group] != null) {
+        var group = thiz.groups[x.group];
+        for(var i = 0; i<group.length; i++)
+          group[i].userDateWindow = me.userDateWindow;
+      }
+    };
+  },
+  
+  
   groups: {},
   
   addGroupDrawCallback: function(x) {
@@ -347,8 +432,8 @@ HTMLWidgets.widget({
         
       for (var i = 0; i < x.shadings.length; i++) {
         var shading = x.shadings[i];
-        var x1 = thiz.normalizeDateValue(x.scale, shading.from).getTime();
-        var x2 = thiz.normalizeDateValue(x.scale, shading.to).getTime();
+        var x1 = thiz.normalizeDateValue(x.scale, shading.from, x.fixedtz).getTime();
+        var x2 = thiz.normalizeDateValue(x.scale, shading.to, x.fixedtz).getTime();
         var left = g.toDomXCoord(x1);
         var right = g.toDomXCoord(x2);
         canvas.save();
@@ -385,7 +470,7 @@ HTMLWidgets.widget({
         
         // get event and x-coordinate
         var event = x.events[i];
-        var xPos = thiz.normalizeDateValue(x.scale, event.date).getTime();
+        var xPos = thiz.normalizeDateValue(x.scale, event.date, x.fixedtz).getTime();
         xPos = g.toDomXCoord(xPos);
         
         // draw line
@@ -489,9 +574,10 @@ HTMLWidgets.widget({
   // months or years we need to convert the UTC date value to a local time
   // value that "looks like" the equivilant UTC value. To do this we add the
   // timezone offset to the UTC date.
-  normalizeDateValue: function(scale, value) {
+  // Don't use in case of fixedtz
+  normalizeDateValue: function(scale, value, fixedtz) {
     var date = new Date(value); 
-    if (scale != "minute" && scale != "hourly") {
+    if (scale != "minute" && scale != "hourly" && scale != "seconds" && !fixedtz) {
       var localAsUTC = date.getTime() + (date.getTimezoneOffset() * 60000);
       date = new Date(localAsUTC);
     }
